// C/C++ File
// AUTHOR:   foton
// FILE:     main.c
// ROLE:     to rule them all
// CREATED:  2017-01-30 21:51:28
// MODIFIED: 2017-02-13 20:30:48

#include "fdf.h"
#include <stdio.h>


t_mat4	*mat4_init(void)
{
	t_mat4	*mat;

	mat = NULL;
	if ((mat = (t_mat4 *)malloc(sizeof(t_mat4))))
	{
		mat->mx[0][0] = 1.f;
		mat->mx[1][1] = 1.f;
		mat->mx[2][2] = 1.f;
		mat->mx[3][3] = 1.f;
	}
	return (mat);
}

t_mat4	*mat4_mult(t_mat4 *m1, t_mat4 *m2)
{
	t_mat4	*m3;
	int 	i;
	int 	j;

	i = 0;
	m3 = NULL;
	if ((m3 = (t_mat4 *)malloc(sizeof(t_mat4))))
	{
		while(i < 4)
		{
			j = 0;
			while (j < 4)
			{
				m3->mx[i][j] = m1->mx[i][0] * m2->mx[0][j] +
								 m1->mx[i][1] * m2->mx[1][j] +
								 m1->mx[i][2] * m2->mx[2][j] +
								 m1->mx[i][3] * m2->mx[3][j];
				j++;
			}
			i++;
		}
	}
	return (m3);
}

void	mat4_z_rot(t_mat4 *m, double angle)
{
	angle = angle * M_1_PI / 180;
	m->mx[0][0] = cos(angle);
	m->mx[0][1] = sin(angle);
	m->mx[0][2] = 0;
	m->mx[1][0] = -(sin(angle));
	m->mx[1][1] = cos(angle);
	m->mx[1][2] = 0;
	m->mx[2][0] = 0;
	m->mx[2][1] = 0;
	m->mx[2][2] = 1;
}

void	mat4_x_rot(t_mat4 *m, double angle)
{
	angle = angle * M_1_PI / 180;
	m->mx[0][0] = 1;
	m->mx[0][1] = 0;
	m->mx[0][2] = 0;
	m->mx[1][0] = 0;
	m->mx[1][1] = cos(angle);
	m->mx[1][2] = sin(angle);
	m->mx[2][0] = 0;
	m->mx[2][1] = -(sin(angle));
	m->mx[2][2] = cos(angle);
}

void	mat4_y_rot(t_mat4 *m, double angle)
{
	angle = angle * M_1_PI / 180;
	m->mx[0][0] = cos(angle);
	m->mx[0][1] = 0;
	m->mx[0][2] = -(sin(angle));
	m->mx[1][0] = 0;
	m->mx[1][1] = 1;
	m->mx[1][2] = 0;
	m->mx[2][0] = sin(angle);
	m->mx[2][1] = 0;
	m->mx[2][2] = cos(angle);
}

void	vec_mat_mult(t_mat4 *m, t_vec *v)
{
	double	vec4;
	double	x;
	double 	y;
	double	z;

	x = v->x * m->mx[0][0] + v->y * m->mx[1][0] + v->z * m->mx[2][0] +
			m->mx[3][0];
	y = v->x * m->mx[0][1] + v->y * m->mx[1][1] + v->z * m->mx[2][1] +
			m->mx[3][1];
	z = v->x * m->mx[0][2] + v->y * m->mx[1][2] + v->z * m->mx[2][2] +
			m->mx[3][2];
	vec4 = v->x * m->mx[0][3] + v->y * m->mx[1][3] + v->z * m->mx[2][3] +
			m->mx[3][3];
	if (vec4 != 1 && vec4 != 0)
	{
		x = x / vec4;
		y = y / vec4;
		z = z / vec4;
	}
	v->x = x;
	v->y = y;
	v->z = z;
}

void	test_line(t_mlx mlx)
{
	t_vec	p1;
	t_vec	p2;
	double	temp;

	p1.color = 0xC100E5;
	p2.color = 0xC100E5;
	for (p2.y = 0; p2.y <= 500; p2.y += 4)
	{
		p1.x = 0;
		p1.y = 0;
		p2.x = 500;
		line(p1, p2, &mlx);
	}
	for (p2.x = 500; p2.x >= 0; p2.x -= 4)
	{
		p1.x = 0;
		p1.y = 0;
		p2.y = 500;
		temp = p2.x;
		line(p1, p2, &mlx);
		p2.x = temp;
	}
	p1.color = 0x1EBF00;
	p2.color = 0x1EBF00;
	for (p2.y = 0; p2.y <= 500; p2.y += 4)
	{
		p1.x = 500;
		p1.y = 0;
		p2.x = 0;
		temp = p2.y;
		line(p1, p2, &mlx);
		p2.y = temp;
	}
	for (p2.x = 0; p2.x <= 500; p2.x += 4)
	{
		p1.x = 500;
		p1.y = 0;
		p2.y = 500;
		temp = p2.x;
		line(p1, p2, &mlx);
		p2.x = temp;
	}
}

void	circle(t_mlx m)
{
	t_vec	p1;
	t_vec	p2;
	double angle;


	p1.color = 0xFFFFFF;
	p2.color = 0xFFFFFF;
	for (int i = 0; i < 360; i += 4)
	{
		p1.x = 250;
		p1.y = 250;
		angle = i * M_PI / 180;
		p2.x = p1.x + 200 * sin(angle);
		p2.y = p1.y + 200 * cos(angle);
		line(p1, p2, &m);
	}
}

void	fill_vec3(double x, double y, double z, t_vec *new, int pos)
{
	new->x = x;
	new->y = y;
	new->z = z;
	new->pos = pos;
	new->color = 0xFFFFFFFF;
}

void	fill_edg(int p1, int p2,  t_edg *edg)
{
	edg->p1 = p1;
	edg->p2 = p2;
}

void	perspective_transform(t_list *vec_list, t_mlx *mlx, t_vec **points)
{
	double	xmin = -0.5;
	double	xmax = 0.5;
	double	ymin = -0.5;
	double	ymax = 0.5;
	double 	x;
	double 	y;
	double 	z;
	t_vec	*point;
	int 	i;

	i = 0;
	while(vec_list)
	{
		x = ((t_vec *)vec_list->content)->x;
		y = ((t_vec *)vec_list->content)->y;
		z = ((t_vec *)vec_list->content)->z;
		x = x / z;
		y = y / z;
		point = (t_vec *)malloc(sizeof(t_vec));
		points[i] = point;
		points[i]->x = 500 * (1 - (x - xmin) / (xmax - xmin));
		points[i]->y = 500 * (1 - (y - ymin) / (ymax - ymin));
		points[i]->color = 0xFFFFFFFF;
		mlx_pixel_put(mlx->mlx, mlx->win, (int)points[i]->x, (int)points[i]->y, 0xFFFFFFFF);
		vec_list = vec_list->next;
		i++;
	}
}

void	edges(t_vec **points, t_list *edg, t_mlx *m)
{
	int p1;
	int p2;
	int i;

	i = 0;
	while (points[i]) {
		printf("point x - %f, y - %f\n", points[i]->x, points[i]->y);
		i++;
	}
	while (edg)
	{
		p1 = ((t_edg *)edg->content)->p1;
		p2 = ((t_edg *)edg->content)->p2;
		line(
				*points[p1],
				*points[p2],
				m
		);
		edg = edg->next;
	}
}

void	transform(t_mat4 *mat, t_list *v_list)
{
	int i;

	i = 0;
	while(v_list)
	{
		vec_mat_mult(mat, v_list->content);
		i++;
		v_list = v_list->next;
	}
}

int		main(int argc, char **argv)
{
	t_mlx	m;
	t_list	*vec_list;
	t_list	*edg_list;
	t_vec	vec;
	t_vec	**points;
	t_edg	edg;
	int 	vec_list_length;
	t_mat4	*mat;

//	int 	fd;
//	char 	*line;
//
//	if (argc != 2)
//		return (0);
//	fd = open(argv[1], O_RDONLY);
//	// add new get_next_line
//	while (get_next_line(fd, &line))
//	{
//		ft_putstr(line);
//		ft_putstr("\n");
//	}

	m.mlx = mlx_init();
	m.win = mlx_new_window(m.mlx, 500, 500, "fdf");
	//test_line(m);
	//circle(m);

	argc = argc;
	argv = argv;
	vec_list = NULL;
	edg_list = NULL;
	fill_vec3(-0.5, -0.5, 2.5, &vec, 0);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));
	fill_vec3(-0.5, 0.5, 2.5, &vec, 1);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));
	fill_vec3(0.5, 0.5, 2.5, &vec, 2);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));
	fill_vec3(0.5, -0.5, 2.5, &vec, 3);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));
	fill_vec3(-0.5, -0.5, 3.5, &vec, 4);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));
	fill_vec3(-0.5, 0.5, 3.5, &vec, 5);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));
	fill_vec3(0.5, 0.5, 3.5, &vec, 6);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));
	fill_vec3(0.5, -0.5, 3.5, &vec, 7);
	ft_lstadd_back(&vec_list, &vec, sizeof(t_vec));

	vec_list_length = ft_lst_len(vec_list);
	points = (t_vec **)malloc(sizeof(t_vec *) * (vec_list_length + 1));
	fill_edg(0, 1, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(1, 2, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(2, 3, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(3, 0, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(0, 4, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(1, 5, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(2, 6, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(3, 7, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(4, 5, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(5, 6, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(6, 7, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));
	fill_edg(7, 4, &edg);
	ft_lstadd_back(&edg_list, &edg, sizeof(int[2]));

	mat = mat4_init();

	mat4_z_rot(mat, 360);
	transform(mat, vec_list);
	perspective_transform(vec_list, &m, points);
	edges(points, edg_list, &m);
	mlx_loop(m.mlx);
	return (0);
}
